==> gkrellm-helium-2.1.tgz: gkrellm-helium-2.1/helium.c <==
/* Gkrellm Helium plugin */

#include <gkrellm2/gkrellm.h>
#include <pthread.h>
#include "misc.h"

#define	CONFIG_NAME	"Helium"

#define	STYLE_NAME	"helium"

static pthread_t receiver;
static pthread_mutex_t mutex;
static char temp_text[2][256];
static volatile int cnt = 0;
static volatile int running = 0;

static GkrellmMonitor *monitor;

static const int panel_cnt = 2;

static GkrellmPanel *panel[2];
static GkrellmDecal *decal_text_label[2], *decal_text_temp[2];

static gint style_id;

static cfg helium_cfg;
static cfg kuna_cfg;

static gint panel_expose_event(GtkWidget * widget, GdkEventExpose * ev, gint idx)
{
    gdk_draw_pixmap(widget->window,
		    widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		    panel[idx]->pixmap, ev->area.x, ev->area.y, ev->area.x,
		    ev->area.y, ev->area.width, ev->area.height);
    return FALSE;
}


static void *receive(void *data)
{
    char helium_therm[7], kuna_therm[7];
    helium_udp_get_actual_value(helium_therm, &helium_cfg);
    helium_http_get_actual_value(kuna_therm, &kuna_cfg);
    pthread_mutex_lock(&mutex);
    sprintf(temp_text[0], "%s C", helium_therm);
    sprintf(temp_text[1], "%s C", kuna_therm);
    cnt++;
    pthread_mutex_unlock(&mutex);
    running = 0;
    return NULL;
}


static void update_plugin()
{
    static gint w, first = 1;
    int i;

    if ((first || GK.minute_tick) && !running) {
	first = 0;
	running = 1;
	pthread_create(&receiver, NULL, receive, NULL);
    }

    if (w == 0)
	w = gkrellm_chart_width();
    for (i = 0; i < panel_cnt; i++) {
	pthread_mutex_lock(&mutex);
	gkrellm_draw_decal_text(panel[i], decal_text_temp[i], temp_text[i], cnt);
	pthread_mutex_unlock(&mutex);
	decal_text_temp[i]->x = w - decal_text_temp[i]->w;
	gkrellm_draw_panel_layers(panel[i]);
    }
}


static void create_plugin(GtkWidget * vbox, gint first_create)
{
    GkrellmStyle *style;
    GkrellmTextstyle *ts, *ts_alt;
    int i;

    strcpy(helium_cfg.server, "147.32.112.33");
    helium_cfg.serverport = 7465;

    strcpy(kuna_cfg.server, "147.32.96.171");
    kuna_cfg.serverport = 80;

    if (first_create) {
	for (i = 0; i < panel_cnt; i++)
	    panel[i] = gkrellm_panel_new0();
    }
    

    style = gkrellm_meter_style(style_id);

    ts = gkrellm_meter_textstyle(style_id);
    ts_alt = gkrellm_meter_alt_textstyle(style_id);

    for (i = 0; i < panel_cnt; i++) {
	decal_text_label[i] =
	    gkrellm_create_decal_text(panel[i], "Helium", ts, style, -1, -1, -1);

	decal_text_temp[i] =
	    gkrellm_create_decal_text(panel[i], "88.8 C", ts_alt, style, -1, -1,
				      0);
	gkrellm_panel_configure(panel[i], NULL, style);
	gkrellm_panel_create(vbox, monitor, panel[i]);
    }

    gkrellm_draw_decal_text(panel[0], decal_text_label[0], "SH", -1);
    gkrellm_draw_decal_text(panel[1], decal_text_label[1], "MK", -1);

    if (first_create) {
	for (i = 0; i < panel_cnt; i++)
	    gtk_signal_connect(GTK_OBJECT(panel[i]->drawing_area), "expose_event",
			       (GtkSignalFunc) panel_expose_event, GINT_TO_POINTER(i));
    }
    
    pthread_mutex_init(&mutex, NULL);
}


static GkrellmMonitor plugin_mon = {
    CONFIG_NAME,		/* Name, for config tab.    */
    0,				/* Id,  0 if a plugin       */
    create_plugin,		/* The create function      */
    update_plugin,		/* The update function      */
    NULL,			/* The config tab create function   */
    NULL,			/* Apply the config function        */

    NULL,			/* Save user config                     */
    NULL,			/* Load user config                     */
    NULL,			/* config keyword                       */

    NULL,			/* Undefined 2  */
    NULL,			/* Undefined 1  */
    NULL,			/* private              */

    MON_MAIL,			/* Insert plugin before this monitor                    */

    NULL,			/* Handle if a plugin, filled in by GKrellM     */
    NULL			/* path if a plugin, filled in by GKrellM       */
};


GkrellmMonitor *gkrellm_init_plugin()
{
    style_id = gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
    monitor = &plugin_mon;
    return &plugin_mon;
}

==> gkrellm-helium-2.3.1.tgz: gkrellm-helium-2.3.1/helium.c <==
/* Gkrellm Helium plugin */

#include <gkrellm2/gkrellm.h>
#include "misc.h"

#define	CONFIG_NAME	"Helium"
#define	STYLE_NAME	"helium"

static GMutex *mutex;
static char temp_text[2][256];
static volatile int cnt = 0;
static volatile int running = 0;

static GkrellmMonitor *monitor;

#define TEMP_CNT 2

static const int panel_cnt = TEMP_CNT;

static GkrellmPanel *panel[TEMP_CNT];
static GkrellmDecal *decal_text_label[TEMP_CNT], *decal_text_temp[TEMP_CNT];

static gint style_id;

static cfg config[TEMP_CNT+1] = 
{
    {
	servertype : SERVER_TYPE_KUNA,
	title : "MK",
	params : {
	kuna : {
	    url : "http://kuna4.mk.cvut.cz/teplota.txt"
	    }
	}
    },
    {
	servertype : SERVER_TYPE_HELIUM,
	title : "SH",
	params : {
	helium : {
	    hostname : "stoupa.sh.cvut.cz",
	    serverport : 7465,
	    }
	}
    },
    {
	servertype : 0
    }
};

static char* get_actual_value(cfg *helium_cfg)
{
    switch (helium_cfg->servertype) {
    case SERVER_TYPE_KUNA:
        return kuna_get_actual_value(helium_cfg);
    case SERVER_TYPE_HELIUM:
        return helium_get_actual_value(helium_cfg);
    default:
	return strdup("Err");
    }
}

static gint panel_expose_event(GtkWidget * widget, GdkEventExpose * ev, gint idx)
{
    gdk_draw_pixmap(widget->window,
		    widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		    panel[idx]->pixmap, ev->area.x, ev->area.y, ev->area.x,
		    ev->area.y, ev->area.width, ev->area.height);
    return FALSE;
}


static gpointer receive(gpointer data)
{
    int i;

    for (i = 0; i < panel_cnt; i++) {
	char *therm = get_actual_value(&config[i]);
	g_mutex_lock(mutex);
	if (therm) {
	    sprintf(temp_text[i], "%s C", therm);
	    free(therm);
	} else {
	    sprintf(temp_text[i], "N/A");
	}
	g_mutex_unlock(mutex);
    }
    running = 0;
    return NULL;
}


static void update_plugin()
{
    static gint w, first = 1;
    int i;

    if ((first || GK.minute_tick) && !running) {
	first = 0;
	running = 1;
	g_thread_create(receive, NULL, FALSE, NULL);
    }

    if (w == 0)
	w = gkrellm_chart_width();
    for (i = 0; i < panel_cnt; i++) {
	g_mutex_lock(mutex);
	gkrellm_draw_decal_text(panel[i], decal_text_temp[i], temp_text[i], cnt);
	g_mutex_unlock(mutex);
	decal_text_temp[i]->x = w - decal_text_temp[i]->w;
	gkrellm_draw_panel_layers(panel[i]);
    }
}


static void create_plugin(GtkWidget * vbox, gint first_create)
{
    GkrellmStyle *style;
    GkrellmTextstyle *ts, *ts_alt;
    int i;

    if (first_create) {
	for (i = 0; i < panel_cnt; i++)
	    panel[i] = gkrellm_panel_new0();
    }

    style = gkrellm_meter_style(style_id);

    ts = gkrellm_meter_textstyle(style_id);
    ts_alt = gkrellm_meter_alt_textstyle(style_id);

    for (i = 0; i < panel_cnt; i++) {
	decal_text_label[i] =
	    gkrellm_create_decal_text(panel[i], "Helium", ts, style, -1, -1, -1);

	decal_text_temp[i] =
	    gkrellm_create_decal_text(panel[i], "88.8 C", ts_alt, style, -1, -1,
				      0);
	gkrellm_panel_configure(panel[i], NULL, style);
	gkrellm_panel_create(vbox, monitor, panel[i]);
    }

    for (i = 0; i < panel_cnt; i++) {
	gkrellm_draw_decal_text(panel[i], decal_text_label[i], config[i].title, -1);
    }
    
    if (first_create) {
	for (i = 0; i < panel_cnt; i++)
	    gtk_signal_connect(GTK_OBJECT(panel[i]->drawing_area), "expose_event",
			       (GtkSignalFunc) panel_expose_event, GINT_TO_POINTER(i));
    }
    
    mutex = g_mutex_new();
}


static GkrellmMonitor plugin_mon = {
    CONFIG_NAME,		/* Name, for config tab.    */
    0,				/* Id,  0 if a plugin       */
    create_plugin,		/* The create function      */
    update_plugin,		/* The update function      */
    NULL,			/* The config tab create function   */
    NULL,			/* Apply the config function        */

    NULL,			/* Save user config                     */
    NULL,			/* Load user config                     */
    NULL,			/* config keyword                       */

    NULL,			/* Undefined 2  */
    NULL,			/* Undefined 1  */
    NULL,			/* private              */

    MON_MAIL,			/* Insert plugin before this monitor                    */

    NULL,			/* Handle if a plugin, filled in by GKrellM     */
    NULL			/* path if a plugin, filled in by GKrellM       */
};


GkrellmMonitor *gkrellm_init_plugin()
{
    style_id = gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
    monitor = &plugin_mon;
    return &plugin_mon;
}
